/*
 * Copyright (c) 1995-2002 Conexant Systems, Inc.
 * 
 * 1.   Permitted use. Redistribution and use in source and binary forms,
 * with or without modification, are permitted under the terms set forth
 * herein.
 * 
 * 2.   Disclaimer of Warranties. CONEXANT AND OTHER CONTRIBUTORS MAKE NO
 * REPRESENTATION ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.
 * IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTIES OF ANY KIND.
 * CONEXANT AND OTHER CONTRIBUTORS DISCLAIMS ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, GOOD TITLE AND AGAINST INFRINGEMENT.
 * 
 * This software has not been formally tested, and there is no guarantee that
 * it is free of errors including, but not limited to, bugs, defects,
 * interrupted operation, or unexpected results. Any use of this software is
 * at user's own risk.
 * 
 * 3.   No Liability.
 * 
 * (a) Conexant or contributors shall not be responsible for any loss or
 * damage to Company, its customers, or any third parties for any reason
 * whatsoever, and CONEXANT OR CONTRIBUTORS SHALL NOT BE LIABLE FOR ANY
 * ACTUAL, DIRECT, INDIRECT, SPECIAL, PUNITIVE, INCIDENTAL, OR CONSEQUENTIAL
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED, WHETHER IN CONTRACT, STRICT OR OTHER LEGAL THEORY OF
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * (b) User agrees to hold Conexant and contributors harmless from any
 * liability, loss, cost, damage or expense, including attorney's fees,
 * as a result of any claims which may be made by any person, including
 * but not limited to User, its agents and employees, its customers, or
 * any third parties that arise out of or result from the manufacture,
 * delivery, actual or alleged ownership, performance, use, operation
 * or possession of the software furnished hereunder, whether such claims
 * are based on negligence, breach of contract, absolute liability or any
 * other legal theory.
 * 
 * 4.   Notices. User hereby agrees not to remove, alter or destroy any
 * copyright, trademark, credits, other proprietary notices or confidential
 * legends placed upon, contained within or associated with the Software,
 * and shall include all such unaltered copyright, trademark, credits,
 * other proprietary notices or confidential legends on or in every copy of
 * the Software.
 * 
 */

/*

  MODULE NAME:     int_mdl.c                       

  AUTHOR:          Konstantin Lazarev    (Transcribed by KNL)

  HISTORY:         Major Revision               Date        By
            -----------------------------     --------     -----
            Created                           10/01/99      KNL

  DESCRIPTION:

    This module is hardware interface child driver for Conexant RipTide.
    
  NOTES:    None.
*/

#include <linux/config.h>
#include <linux/string.h>
#include <linux/ioport.h>
#include <linux/sched.h>
#include <linux/wait.h>
#include <linux/delay.h>
#include <linux/sound.h>
#include <linux/slab.h>
#include <linux/soundcard.h>
#include <linux/pci.h>
#include <asm/io.h>
#include <asm/dma.h>
#include <linux/init.h>
#include <linux/poll.h>
#include <linux/module.h>
/*#include <asm/spinlock.h>*/
#include <asm/uaccess.h>
#include <asm/hardirq.h>
#include <linux/wait.h>
#include <linux/kmod.h>

#include "cnxt_int.h"
//#include "lowlevelrp_cmdif.h"

HwContext *devs = NULL;

/* --------------------------------------------------------------------- */
/*************************************************************************
 *  Name:           void release_device(rt_state *phwInfo)
 *
 *  Parameters:     phwInfo - driver state structure
 *
 *  Returns:        None
 *
 *  Description:
 *    The function release driver resources
 *
 *************************************************************************/
void release_device(HwContext *phwInfo)
{
    printk("<RTINT> Delete device: %p\n", phwInfo);
    if(!phwInfo)
    {
        printk("<RTINT> No device descriptor.\n");
        return;
    }

    if(phwInfo->ioPort)
        release_region(phwInfo->ioPort, 64);
    else
        printk("<RTINT> No io region to release\n");

    kfree_s(phwInfo, sizeof(HwContext));
    printk("<RTINT> Device resources are released\n");
}

HwContext *cnxt_rt_open(int ioPort, int deviceId)
{
HwContext *p = devs, *ret = NULL;

    while(p != NULL)
    {
        if(p->ioPort == ioPort)
        {
            ret = p;
            break;
        }  
        p = p->next;
    }
    if(p == NULL)
    {
        if (!(p = kmalloc(sizeof(HwContext), GFP_KERNEL)))
        {
            printk(KERN_ERR"<RTINT> out of memory\n");
        }
        else
        {
            p->ioPort = ioPort;
            p->pInterface = NULL;
            if((p->pfunc = (voidfunc)RiptideInitialize(&p->pInterface, deviceId, ioPort)) == NULL)
            {  
                printk(KERN_ERR"<RTINT> Can't init interface\n");
            }
            else
            {
                init_spinlock(&p->sysObj);
                if (check_region(ioPort, 64))
                {
                    printk(KERN_ERR "<RTINT> io ports %#x-%#x in use\n", ioPort, ioPort+63);
                }
                else
                {
/* allocate region for I/O ports */
                    printk(KERN_INFO "<RTINT> requesting io ports %#x-%#x\n", ioPort, ioPort+63);
                    request_region(p->ioPort, 64, "riptide");
                    ret = p;
                    p->next = devs;
                    devs = p;
                }
            }
        }
    }
    if(ret != NULL)
        MOD_INC_USE_COUNT;
    else
        release_device(p);
    return ret;
}

void cnxt_rt_close(HwContext *pContext)
{
HwContext *p = devs;
HwContext **pp = &devs;

    while(p != NULL)
    {
        if(p == pContext)
        {
            *pp = p->next;
            release_device(pContext);
            break;
        }
        pp = &(p->next);
        p = p->next;
    }
    if(p != NULL)
        MOD_DEC_USE_COUNT;
}

/*************************************************************************
 *  Name:           int init_module(void)
 *
 *  Parameters:     none
 *
 *  Returns:        None
 *
 *  Description:
 *    The function initializes module, finds device, inits device and allocate resources
 *
 *************************************************************************/
MODULE_AUTHOR("Conexant Systems Inc.");
MODULE_DESCRIPTION("RipTide Hardware Interface Driver");
MODULE_LICENSE("Copyright (C) 1995-2002 Conexant Systems Inc. All Rights Reserved.");

EXPORT_SYMBOL_NOVERS(cnxt_rt_open);
EXPORT_SYMBOL_NOVERS(cnxt_rt_close);

int init_module(void)
{
    return 0;
}

void cleanup_module(void)
{
}
